require 'spec_helper'
require 'emcee/runner'

describe Emcee::Runner do
  describe 'initializing' do
    it 'works without arguments' do
      lambda { Emcee::Runner.new }.should_not raise_error(ArgumentError)
    end
  end

  describe 'returning the set of results available' do
    before do
      @runner = Emcee::Runner.new
      @results = fake_results.collect {|node_data| stub(:results => node_data) }
      @runner.stubs(:raw_agent_inventory).returns(@results)
    end

    it 'returns the union of result information over all nodes from a raw inventory' do
      @runner.results.should == fake_results
    end
  end

  describe 'running an action on the collective' do
    before do
      @runner = Emcee::Runner.new
      @agent_name = 'someagent'
      @action = stub(:name => @agent_name)
      @args = { :one => 2, :three => 4 }
      @client = 'fake client'
      @result = 'fake client result'
      @client.stubs(:custom_request).returns(@result)
      @client.stubs(:disconnect)
      @filtered_nodes = ['node1', 'node2']
      @node_filter = 'somefilter'
      @runner.stubs(:filtered_nodes).returns(@filtered_nodes)
      @runner.stubs(:node_filter).returns(@node_filter)
      @runner.stubs(:client_for).returns(@client)
    end

    it 'runs the named action with arguments via a custom request on the RPC client with the relevant node list' do
      @client.expects(:custom_request).with(@agent_name, @args, @filtered_nodes, { 'identity' => @node_filter })

      @runner.run(@action, @args)
    end

    it 'returns the result of the RPC client call' do
      @runner.run(@action, @args).should == @result
    end
  end

  describe 'finding the RPC client for an action' do
    before do
      @client = stub_everything
      @action_name = 'testit'
      @interface = {
        :output => { :output   => { :display_as => "Output", :description => "Output from the Nagios plugin" },
          :exitcode => { :display_as => "Exit Code", :description => "Exit Code from the Nagios plugin" },
          :perfdata => { :display_as => "Performance Data", :description => "Performance Data from the Nagios plugin" }
      },
        :action      => @action_name,
        :description => "Run a NRPE command",
        :display => :failed,
        :input => {
        :command => {
        :type        => :string,
        :description => "NR PE command to run",
        :prompt      => "Command",
        :optional    => false,
        :validation  => "^[a-zA-Z0-9_-]+$",
        :maxlength   => 50
      }
      }
      }
      @agent_name = 'someagent'
    end

    describe 'when no filter is specified' do
      before do
        @runner = Emcee::Runner.new
        @agent = Emcee::Agent.new(@runner, :agent => @agent_name, :version => '1.0')
        @action = Emcee::Action.new(@agent, @interface)
        @runner.stubs(:rpcclient).returns(@client)
      end

      it 'accepts an action object' do
        lambda { @runner.client_for(@action) }.should_not raise_error(ArgumentError)
      end

      it 'requires an action object' do
        lambda { @runner.client_for }.should raise_error(ArgumentError)
      end

      it 'looks up client handle for the agent for the supplied action' do
        @runner.expects(:client_with_timeout).with(@agent_name)
        @runner.client_for(@action)
      end

      it 'does not apply fact filters' do
        @client.expects(:fact_filter).never
        @runner.client_for(@action)
      end

      it 'does not apply class filters' do
        @client.expects(:class_filter).never
        @runner.client_for(@action)
      end

      it 'does not apply identity filters' do
        @client.expects(:identity_filter).never
        @runner.client_for(@action)
      end

      it 'returns the client handle for the agent for the supplied action' do
        @runner.client_for(@action).should == @client
      end
    end

    describe 'when a filter is specified' do
      before do
        @filter = { :facts      => [ [ 'foo' , 'bar' ], [ 'baz' , 'xyzzy' ] ],
                    :classes    => [ 'fooclass', 'barclass' ],
                    :identities => [ 'foonode', /barnode/ ] }
        @runner = Emcee::Runner.new(:filter => @filter)
        @agent = Emcee::Agent.new(@runner, :agent => @agent_name, :version => '1.0')
        @action = Emcee::Action.new(@agent, @interface)
        @runner.stubs(:rpcclient).returns(@client)
      end

      it 'accepts an action object' do
        lambda { @runner.client_for(@action) }.should_not raise_error(ArgumentError)
      end

      it 'requires an action object' do
        lambda { @runner.client_for }.should raise_error(ArgumentError)
      end

      it 'looks up client handle for the agent for the supplied action' do
        @runner.expects(:rpcclient).with(@agent_name).returns(@client)
        @runner.client_for(@action)
      end

      it 'applies fact filters when available' do
        @client.expects(:fact_filter).with('foo', 'bar').with('baz', 'xyzzy')
        @runner.client_for(@action)
      end

      it 'applies class filters when available' do
        @client.expects(:class_filter).with('fooclass').with('barclass')
        @runner.client_for(@action)
      end

      it 'applies identity filters when available' do
        @client.expects(:identity_filter).with('foonode').with(/barnode/)
        @runner.client_for(@action)
      end

      it 'returns the client handle for the agent for the supplied action' do
        @runner.client_for(@action).should == @client
      end
    end
  end

  describe 'returning the list of filtered nodes' do
    before do
      @runner = Emcee::Runner.new
      @filtered_nodes = [ 'node1', 'node2', 'node3' ]
      @fake_client = stub_everything
      @filtered_client = stub(:discover => @filtered_nodes, :disconnect => nil)
      @runner.stubs(:rpcclient).returns(@fake_client)
      @runner.stubs(:filtered_client).returns(@filtered_client)
    end

    it 'gets an mcollective rpcutil client' do
      @runner.expects(:rpcclient).with('rpcutil').returns(@fake_client)
      @runner.filtered_nodes
    end

    it 'applies the specified filters to the mcollective client' do
      @runner.expects(:filtered_client).with(@fake_client).returns(@filtered_client)
      @runner.filtered_nodes
    end

    it 'returns the discovery list from the filtered mcolective client' do
      @runner.filtered_nodes.should == @filtered_nodes
    end
  end

  describe 'providing the DDL for an agent' do
    before do
      @runner = Emcee::Runner.new
      @ddl = {}
      @mock_client = stub(:ddl => @ddl)
      @runner.stubs(:client_with_timeout).returns(@mock_client)
    end

    it 'accepts an agent name' do
      lambda { @runner.ddl('agent') }.should_not raise_error(ArgumentError)
    end

    it 'requires an agent name' do
      lambda { @runner.ddl }.should raise_error(ArgumentError)
    end

    it 'returns nil if no DDL can be found for the agent' do
      @mock_client = stub(:ddl => nil)
      @runner.stubs(:client_with_timeout).returns(@mock_client)
      @runner.ddl('notfound').should be_nil
    end

    it 'returns a DDL instance if a DDL can be found for the agent' do
      @runner.ddl('found').should == @ddl
    end
  end

  describe '#nodeset_to_identity' do

    before :all do
      @runner = Emcee::Runner.new
      @nodes = ['foo.bar.baz', 'example.com', 'static.media.example.com']
      @not_nodes = ['foodbar.baz', 'foo.bardbaz', 'bar.baz', 'static.example.com', 'foo.example.com']
      @regex_string = @runner.send(:nodeset_to_identity, @nodes)
      @regex = Regexp.new(@regex_string[1..-2])
    end

    it 'should return a regex string' do
      chars = @regex_string.split('')
      chars.first.should == '/'
      chars.last.should == '/'
    end

    it 'should return a regex string that matches each of the input nodes' do
      @nodes.each do |n|
        n.should match @regex
      end
    end

    it 'should return a regex string that does not match anything other than the input nodes' do
      @not_nodes.each do |n|
        n.should_not match @regex
      end
    end
  end

  describe '#client_with_timeout' do
    it 'should look up an rpcclient with the passed name and set its discovery timeout' do
      runner = Emcee::Runner.new
      client = stub
      runner.expects(:rpcclient).with("foo").returns(client)
      client.expects(:discovery_timeout=).with(4)
      runner.client_with_timeout("foo")
    end
  end
end

